@@ -67,9 +67,12 @@ def _sample_to_alignment_files(sample_to_vcf_file_dict: dict, RG_ID_dict: dict)
     """
     sample_to_alignment_files = {}
     for sample in sample_to_vcf_file_dict:
+        #sys.stderr.write("%s\n" % sample)
         sample_to_alignment_files[sample] = {}
         for file in RG_ID_dict.keys():
+            #sys.stderr.write("%s\n" % file)
             if len([file for key in RG_ID_dict[file].keys() if RG_ID_dict[file][key]['SM'] == sample]) > 0:
+                #sys.stderr.write("%s\n" % file)
                 for pair in [{
                         alignment_file_path:any(
                                 [RG_ID_dict[alignment_file_path][ID]['RG_tags'] for ID in
@@ -79,6 +82,7 @@ def _sample_to_alignment_files(sample_to_vcf_file_dict: dict, RG_ID_dict: dict)
                         [RG_ID_dict[alignment_file_path][ID]['SM'] == sample for ID in
                          RG_ID_dict[alignment_file_path].keys()]
                         )]:
+                    #sys.stderr.write("%s\n" % pair)
                     sample_to_alignment_files[sample][list(pair.keys())[0]] = list(pair.values())[0]
     return sample_to_alignment_files
 
@@ -394,7 +398,7 @@ class InputData:
             _prepare_output_directory(
                     self.output_directory + '/reference_sequences/' + str(reference_sequence_input_assembly)
                     )
-        
+
         if not self.long_read_input is None:
             # self.long_read_input = long_read_input
             long_read_input_path, input_ID, input_sample, input_sample_description, input_reference_sequence_input = _extract_RG_info_from_long_read_input(long_read_input)
@@ -428,6 +432,7 @@ class InputData:
         Stores absolute path to the reference sequence file in self.reference_sequence_files[].
         Returns absolute path to current reference sequence file.
         """
+        #sys.stderr.write("%s; %s; %s\n" % (reference_sequence_input, output_directory, reference_sequence_input_assembly))
 
         # First, build the directory path where the reference sequence file will be written.
         if output_directory is None:
@@ -452,7 +457,7 @@ class InputData:
         # and applies the .fa extension. Could we just test for different file types
         # instead?
         #reference_sequence_input_path = os.path.dirname(os.path.abspath(reference_sequence_input)) + '/' + os.path.basename(reference_sequence_input).split('.')[0] + '.fa'
-        reference_sequence_input_path = reference_sequence_input.split('.')[0] + '.fa'  # Does same thing with less ops
+        reference_sequence_input_path = '.'.join(reference_sequence_input.split('.')[0:-1]) + '.fa'  # Does same thing with less ops
 
         # Instantiate the pysam.FastaFile object given the supplied/downloaded file. Assign
         # this object to self.reference_sequence_files, indexed by the file path.
@@ -492,7 +497,7 @@ class InputData:
 
         # Store the reference sequence path to self.reference_sequence_paths[].
         if self.reference_sequence_paths is None:
-            self.reference_sequence_paths = reference_sequence_file_path
+            self.reference_sequence_paths = [reference_sequence_file_path]
         elif isinstance(self.reference_sequence_paths, str):
             _reference_sequence_paths = []
             _reference_sequence_paths.append(self.reference_sequence_paths)
@@ -550,6 +555,7 @@ class InputData:
                 sample, self.sample_to_vcf_file_dict, self.ignore_phase_sets
                 )
         reference_sequence_names: list = self.sample_to_reference_sequences_dict[sample]
+        #sys.stderr.write("%s\n" % sample)
         #sys.stderr.write("%s\n" % self.sample_to_alignment_files[sample])
         phasable_sample = PhasableSample.PhasableSample(
             sample, vcf_file_path, ignore_phase_sets, self.sample_to_alignment_files[sample], self.RG_ID_dict,
@@ -600,7 +606,6 @@ class InputData:
         except Exception as e:
             sys.stderr.write("Error parsing long-read file: %s\n" % e.strerror)
             exit(2)
-            
         if combined_long_read_fastq_path:
             if reference_sequence_input is None and self.reference_sequence_input is None:
                 sys.stderr.write("Must supply reference sequence!\n")
@@ -624,14 +629,19 @@ class InputData:
                 sorted_bam_file_path, combined_long_read_fastq_path = self._parse_long_reads_input(
                         _sorted_bam_file_path, self.output_directory
                         )
+                #sys.stderr.write("%s; %s\n" % (sorted_bam_file_path, combined_long_read_fastq_path))
                 self.RG_ID_dict, self.unique_RG_IDs = _compile_read_groups(
-                        sorted_bam_file_path[0], sample, ID, sample_description, self.RG_ID_dict, self.unique_RG_IDs,
-                        self.ignore_samples
-                        )
+                    sorted_bam_file_path[0], sample, ID, sample_description,
+                    self.RG_ID_dict, self.unique_RG_IDs,
+                    self.ignore_samples
+                )
+                #sys.stderr.write("%s; %s\n" % (self.RG_ID_dict, self.unique_RG_IDs))
                 self.alignment_files.append(sorted_bam_file_path[0])
+                #sys.stderr.write("%s\n" % self.alignment_files)
                 self.sample_to_alignment_files = _sample_to_alignment_files(
                         self.sample_to_vcf_file_dict, self.RG_ID_dict
                         )
+                #sys.stderr.write("%s\n" % self.sample_to_alignment_files)
                 self._sample_to_reference_sequences_dict(sorted_bam_file_path[0])
                 self._sample_to_PG_dict(sam_file_path = sorted_bam_file_path[0])
                 if reference_sequence_input is not None:
