@@ -734,10 +734,11 @@ def _phasing_decision(
             max_phase = 0
         else:
             max_phase = log_likelihood_ratios.index(max_log_likelihood_ratio) + 1
-            if powlaw_modified(max_log_likelihood_ratio, a=powlaw_alpha, xmin=powlaw_xmin) <= error_rate_threshold:
+#AB - this is a hard coded use of error rate instead of log likihood
+#            if powlaw_modified(max_log_likelihood_ratio, a=powlaw_alpha, xmin=powlaw_xmin) <= error_rate_threshold:
                 # if max_log_likelihood_ratio >= 10 ** (math.log10((error_rate_threshold / .5491)) / -2.45):
-                phase = max_phase
-            else:
-                phase = 'Unphased'
+            phase = max_phase
+#            else:
+#                phase = 'Unphased'
         
         return [log_likelihood_ratios, phase, max_phase, max_log_likelihood_ratio]
