@@ -1,19 +1,13 @@
 # coding=utf-8
-import os
+import sys, os, re
 import subprocess
 import time
-
+from glob import glob
 import pyliftover
-import pysam
-from pysam import bcftools
-from pysam import faidx
+from pysam import faidx, bcftools, FastxFile, AlignedSegment
 from typing import Tuple, List
 
 
-#import pysam.bcftools
-#import pysam.samtools
-
-
 def simulate_reads_pbsim2(
         reference_sequence: str, path_to_pbsim: str = 'pbsim', depth: int = 1,
         simulation_mode: str = 'pbsim2/data/R103.model',
@@ -22,7 +16,7 @@ def simulate_reads_pbsim2(
         length_sd: int = 15000, accuracy_min: float = 0.01,
         accuracy_max: float = 1.00, accuracy_mean: float = 0.80,
         prefix: str = None, id_prefix: str = 'S', output_directory: str = None, sample: str = None,
-        haplotype: int = None
+        haplotype: int = None, quiet = False, silent = False
         ) -> str:
     """
     Args:
@@ -59,8 +53,6 @@ def simulate_reads_pbsim2(
         os.mkdir(output_directory)
     
     if not sample is None and not haplotype is None:
-        #id_prefix = str(sample)+'__'+str(haplotype)+'__'
-
         if prefix is None:
             prefix = str(output_directory) + '/' + str(sample) + '__' + str(haplotype) + '__' + 'pbsim2_simulated_' +\
                      os.path.basename(reference_sequence).split('.')[0] + '_' + str(time.localtime()[0]) + '_' + str(
@@ -72,11 +64,6 @@ def simulate_reads_pbsim2(
             prefix = '{0}/{1}__{2}__{3}'.format(str(output_directory), str(sample), str(haplotype), str(prefix))
     else:
         prefix = '{0}/{1}__{2}__{3}'.format(str(output_directory), str(sample), str(haplotype), str(prefix))
-    # if prefix is None:
-    # if prefix is None:
-    # prefix = str(output_directory) + '/pbsim2_simulated'
-    # else:
-    # prefix = str(output_directory) + '/' + str(prefix)
     
     if str(simulation_mode).lower().endswith(".fastq"):
         mode1 = '--sample-fastq'
@@ -87,12 +74,11 @@ def simulate_reads_pbsim2(
         mode2 = simulation_mode
     
     if not reference_sequence:
-        print(
-                f'no reference genome was provided for pbsim2 simulation. Please specify reference_genome_path to be '
-                f'simulated '
-                )
+        sys.stderr.write('No reference genome was provided for pbsim2 simulation. Please specify reference_genome_path.\n')
+        raise exception('Reference sequence not provided.')
         return
-    print('################ Beginning Read Simulation ################')
+    
+    sys.stderr.write('\n################ Beginning Read Simulation ################\n')
     
     start_process_time = time.time()
     
@@ -100,83 +86,117 @@ def simulate_reads_pbsim2(
     reference_genome_index_path = str(reference_genome_path) + '.fai'
     if not os.path.isfile(reference_genome_index_path):
         faidx(reference_genome_path)
-        #subprocess.run(['samtools', 'faidx', reference_genome_path], check=True)
-    
-    subprocess.run(
-            [str(path_to_pbsim), f'--depth', str(depth),
-             str(mode1), str(mode2),
-             f'--difference-ratio', str(difference_ratio),
-             f'--length-mean', str(length_mean),
-             f'--length-min', str(length_min),
-             f'--length-max', str(length_max),
-             f'--length-sd', str(length_sd),
-             f'--accuracy-mean', str(accuracy_mean),
-             f'--accuracy-min', str(accuracy_min),
-             '--accuracy-max', str(accuracy_max), 
-             f'--prefix', str(prefix),
-             '--id-prefix', str(id_prefix),
-             str(reference_genome_path)], check = True
-            )
-    
-    end_process_time = time.time()
-    total_process_time = end_process_time - start_process_time
-    
-    print(f'Simulation finished in {total_process_time:.2f} seconds', '\n')
-    sample_hap_to_true_alignment_dict = {}
-    pbsim2_pbsim2fq_output = str(prefix) + '_pbsim2fq.fastq'
-    subprocess.run(
-            ["paftools.js pbsim2fq %s %s*.maf >> %s" % (
-                    str(reference_genome_index_path), str(prefix), str(pbsim2_pbsim2fq_output))], shell = True
-            )
-    
-    #subprocess.run(['rm %s/*.maf' % os.path.dirname(pbsim2_pbsim2fq_output)], shell=True)
-    #subprocess.run(['rm %s/*.ref' % os.path.dirname(pbsim2_pbsim2fq_output)], shell=True)
-    
-    combined_simulated_long_reads_fastq_path = str(
-            os.path.dirname(pbsim2_pbsim2fq_output)
-            ) + '/combined_simulated_long_reads.fastq'
-    final_combined_simulated_long_reads_fastq_path: str = '{0}/{1}.fastq'.format(
-            str(
-                    os.path.dirname(combined_simulated_long_reads_fastq_path)
-                    ), str(os.path.splitext(os.path.basename(reference_sequence))[0])
-            )
-    #output_fastq = str(final_combined_simulated_long_reads_fastq_path) + '.gz'
-    
-    for file in os.listdir(os.path.dirname(pbsim2_pbsim2fq_output)):
-        file = str(os.path.dirname(pbsim2_pbsim2fq_output)) + '/' + str(file)
-        if str(file).lower().endswith('pbsim2fq.fastq') or str(file).lower().endswith('pbsim2fq.fastq.gz'):
-            continue
-        if str(file).lower().endswith('.maf'):
-            os.remove(file)
-        if str(file).lower().endswith('.ref'):
-            os.remove(file)
-        elif str(file).lower().endswith('.fastq'):
-            with open(combined_simulated_long_reads_fastq_path, 'a') as outfile:
-                with open(file, 'r') as infile:
-                    outfile.write(infile.read())
-            os.remove(file)
-    
-    for read in pysam.FastxFile(pbsim2_pbsim2fq_output):
-        sample_hap_to_true_alignment_dict[str(read.name.split('!')[0])] = read.name
-    
-    os.remove(pbsim2_pbsim2fq_output)
-    
-    with pysam.FastxFile(combined_simulated_long_reads_fastq_path) as in_pysam_fastx:
-        with open(final_combined_simulated_long_reads_fastq_path, mode = 'w') as out_pysam_fastx:
-            for read in in_pysam_fastx:
-                try:
-                    read.name = str(sample) + '__' + str(haplotype) + '__' + str(
-                            sample_hap_to_true_alignment_dict[read.name]
-                            )
-                    out_pysam_fastx.write(str(read) + '\n')
-                except Exception as e:
-                    print(e)
-    
-    os.remove(combined_simulated_long_reads_fastq_path)
-    #final_combined_simulated_long_reads_fastq_path
+
+    pbsim2_args = [str(path_to_pbsim), f'--depth', str(depth),
+                   str(mode1), str(mode2),
+                   f'--difference-ratio', str(difference_ratio),
+                   f'--length-mean', str(length_mean),
+                   f'--length-min', str(length_min),
+                   f'--length-max', str(length_max),
+                   f'--length-sd', str(length_sd),
+                   f'--accuracy-mean', str(accuracy_mean),
+                   f'--accuracy-min', str(accuracy_min),
+                   '--accuracy-max', str(accuracy_max),
+                   f'--prefix', str(prefix),
+                   '--id-prefix', str(id_prefix),
+                   str(reference_genome_path)]
+
+    if silent:
+        subprocess.run(pbsim2_args, check = True, stdout = subprocess.DEVNULL, stderr = subprocess.DEVNULL)
+    elif quiet:
+        subprocess.run(pbsim2_args, check = True, stdout = subprocess.DEVNULL)
+    else:
+        subprocess.run(pbsim2_args, check = True)
+
+    sys.stderr.write('Post-processing simulated fastq files...\n')
+    sample_hap_to_true_alignment_dict = build_true_alignment_dict(reference_genome_index_path, prefix)
+
+    final_combined_simulated_long_reads_fastq_path = '{0}/{1}.fastq'.format(
+        str(output_directory),
+        str(os.path.splitext(os.path.basename(reference_sequence))[0])
+    )    
+    for fastq_file in glob(str(prefix) + '*.fastq'):
+        if not (quiet or silent):
+            sys.stderr.write("%s\n" % fastq_file)
+        with FastxFile(fastq_file, 'r') as infile:
+            with open(final_combined_simulated_long_reads_fastq_path, 'a') as outfile:
+                for read in infile:
+                    read.name = str(sample) + '__' + str(haplotype) + '__' + str(sample_hap_to_true_alignment_dict[read.name])
+                    outfile.write(str(read) + '\n')
+        os.remove(fastq_file)
+
+    # Clean up pbsim2 output files
+    for f in glob(str(prefix) + '*.maf'):
+        os.remove(f)
+    for f in glob(str(prefix) + '*.ref'):
+        os.remove(f)
+
+    sys.stderr.write('Simulation finished in %.2f seconds.\n' % (time.time() - start_process_time))
     return final_combined_simulated_long_reads_fastq_path
 
 
+def build_true_alignment_dict(reference_genome_index_path, prefix):
+    """
+    Build the true alignment dict from pbsim2 maf files.
+    """
+    true_alignment_dict = {}
+    files = glob(prefix + '*.maf')
+    for pbsim2_maf in files:
+        al_dict = extract_true_alignments_from_pbsim2_maf(reference_genome_index_path, pbsim2_maf)
+        true_alignment_dict.update(al_dict)
+    return true_alignment_dict
+
+
+def extract_true_alignments_from_pbsim2_maf(reference_genome_index_path, pbsim2_maf):
+    """
+    Parse true alignment coordinates from a pbsim2 MAF file.
+    """
+    # Build the chromosome list from the reference genome index
+    chr_list = []
+    with open(reference_genome_index_path, 'r') as index_file:
+        for rec in index_file:
+            line = rec.strip('\n').split()
+            if len(line) > 0:
+                chr_list.append(line[0])
+
+    # Parse MAF records, extracting the true alignment coordinates for each simulated read.
+    true_alignment_dict = {}
+    state = 0
+    reg = []
+    with open(pbsim2_maf) as maf_file:
+        for rec in maf_file:
+            line = rec.strip('\n').split()
+            if len(line) == 0:
+                continue
+            elif state == 0 and line[0] == 'a':
+                state = 1
+            elif state == 1 and line[0] == 's':
+                reg = [line[2], line[2]+line[3]]
+                state = 2
+            elif state == 2 and line[0] == 's':
+                m = re.match('^S\d+_\d+', line[1])
+                if not m:
+                    sys.stderr.write("Failed to parse read name from record: %s. Skipping...\n" % " ".join(line[0:5]))
+                    state = 0
+                    continue
+                chr_id = int(line[1].split('S')[1].split('_')[0]) - 1
+                if chr_id > len(chr_list):
+                    sys.stderr.write("Chromosome not in index: %s. Skipping...\n" % str(chr_id+1))
+                    state = 0
+                    continue
+                name = "!".join([line[1], chr_list[chr_id], reg[0], reg[1], line[4]])
+                seq = line[6].replace('-', '')
+                if len(seq) != int(line[5]):
+                    sys.stderr.write("Inconsistent read length for sequence %s. Skipping...\n" % name)
+                    state = 0
+                    continue
+                true_alignment_dict[line[1]] = name
+                #sys.stderr.write("%s, %s\n" % (line[1], name))
+                state = 0
+
+    return true_alignment_dict
+    
+
 def convert_coordinates(
         haplotype: int, sample: str, contig: str, position: int, reference_sequence: str,
         reference_liftover_converters = None, chain_file = None
@@ -244,23 +264,21 @@ def generate_haplotype_specific_fasta(
     Returns:
 
     """
-    print('################ Generating Haplotype Reference Sequence ################')
+    sys.stderr.write('\n################ Generating Reference Sequence for Haplotype %d ################\n' % haplotype)
     input_reference_sequence_path = os.path.abspath(input_reference_sequence_path)
     input_reference_sequence_index_path: str = str(input_reference_sequence_path) + '.fai'
-    
+
     if output_reference_sequence_path is None:
         output_reference_sequence_path: str = str(os.path.splitext(input_reference_sequence_path)[0]) + '_hap' + str(
                 haplotype
                 ) + '_' + str(sample) + '.fa'
-        output_reference_sequence_path_index = str(os.path.splitext(input_reference_sequence_path)[0]) + '_hap' + str(
-                haplotype
-                ) + '_' + str(sample) + '.fa.fai'
     
     if chain_file_path is None:
         chain_file_path = str(os.path.splitext(input_reference_sequence_path)[0]) + '_hap' + str(haplotype) + '_' + str(
                 sample
                 ) + '.chain'
-        
+
+    # This causes python to segfault if bcftools encounters an error!
     fasta_output = bcftools.consensus(
             '-H', str(haplotype), '-s', str(sample), '-c', str(chain_file_path), '-f',
             str(input_reference_sequence_path),
@@ -268,7 +286,8 @@ def generate_haplotype_specific_fasta(
             )
     with open(output_reference_sequence_path, 'w') as output_fasta:
         output_fasta.write(fasta_output)
-    
+
+    sys.stderr.write("Done.\n")
     return output_reference_sequence_path, chain_file_path
 
 
@@ -300,21 +319,23 @@ def index_fasta_file(fasta_file_path: str) -> str:
 
 def create_fasta_file(
         input_fasta_file_path: str, output_fasta_file_path: str = None, regions: List[str] = None, only_autosomal:
-        bool = False
+        bool = False, chrom_sizes: str = None
         ) -> str:
-    f"""
+    """
 
     Args:
         input_fasta_file_path: 
         only_autosomal (object): 
         regions:
-            List of regions and/or contigs ie: ['chr1:10000000:60000000','chr2']
+            List of regions and/or contigs ie: ['chr1:10000000:60000000',...]
         output_fasta_file_path (object): 
     """
+    regions_file: str = str(os.path.splitext(output_fasta_file_path)[0]) + '_regions.txt'
     autosomal_reference_names = ['chr1', 'chr2', 'chr3', 'chr4', 'chr5', 'chr6', 'chr7', 'chr8', 'chr9', 'chr10',
                                  'chr11', 'chr12', 'chr13', 'chr14', 'chr15', 'chr16', 'chr17', 'chr18', 'chr19',
-                                 'chr20', 'chr21', 'chr22']
+                                 'chr20', 'chr21', 'chr22'] # This should be generalized for non-human genomes
     input_fasta_file_path = os.path.abspath(input_fasta_file_path)
+    
     if output_fasta_file_path is None:
         if only_autosomal:
             output_fasta_file_path = str(
@@ -323,27 +344,65 @@ def create_fasta_file(
         else:
             output_fasta_file_path = str(os.path.splitext(input_fasta_file_path)[0]) + 'output_regions.fa'
     
-    if regions is None:
-        regions_file = str(os.path.splitext(output_fasta_file_path)[0]) + '_regions.txt'
-        all_regions = pysam.FastaFile(input_fasta_file_path).references
-        if only_autosomal:
-            regions_list = [region for region in all_regions if region in autosomal_reference_names]
-        else:
-            regions_list = all_regions
-    
-    elif isinstance(regions, list):
-        regions_file: str = str(os.path.splitext(output_fasta_file_path)[0]) + '_regions.txt'
-        if only_autosomal:
-            regions_list = [region for region in regions if region in autosomal_reference_names]
-        else:
-            regions_list = regions
-    
+    if regions is None and not only_autosomal:
+        """
+        If we are not filtering out any specific regions, just test for a fai file
+        and create one if not found. Return the output_fasta_file_path.
+        """
+        if not os.path.isfile(input_fasta_file_path + '.fai'):
+            # This always creates the index in the directory where the fasta file lives.
+            # Don't create a copy of the reference if we're not filtering anything!
+            faidx(input_fasta_file_path, '-o', output_fasta_file_path)
+        return input_fasta_file_path
+
+    # Otherwise, we need to filter out sequences we want and reindex the resulting fasta.
+    if regions is None and only_autosomal:
+        # Explicitly providing regions will always supercede only_autosomal
+        if chrom_sizes == None:
+            # We need chrom.sizes because the regions file must have start and end coordinates. See http://www.htslib.org/doc/samtools-faidx.html
+            raise Exception("ERROR: create_fasta_file only_autosomal requires chrom_sizes.")
+            return
+        with open(regions_file, 'w') as rg_file:
+            with open(chrom_sizes, 'r') as cs_file:
+                for line in cs_file:
+                    rec = line.split()
+                    if rec[0] in autosomal_reference_names:
+                        # Write a line to the regions file.
+                        rg_file.write("{}:1-{}\n".format(rec[0], rec[1]))
+
     elif os.path.isfile(regions):
         regions_file = regions
         
-    with open(regions_file, 'w') as file:
-        for region in regions_list:
-            file.write(str(region) + '\n')
+    elif isinstance(regions, list):
+        if only_autosomal:
+            regions_list = []
+            for region in regions:
+                chrom = region.split(':')[0]
+                if chrom in autosomal_reference_names:
+                    regions_list.append(region)                
+        else:
+          regions_list = regions
+            
+        # Validate all entries in regions_list
+        chrom_sizes_dict = {}
+        with open(chrom_sizes, 'r') as cs_file:
+            for line in cs_file:
+                rec = line.split()
+                chrom_sizes_dict[rec[0]] = rec[1]
+
+        with open(regions_file, 'w') as file:
+            for region in regions_list:
+                # Quick and dirty validation -- just checks for presence of the proper delimiters
+                # and end coordinate within the extent of the chromosome.
+                s1 = region.split(':')
+                s2 = region.split('-')
+                if len(s1) == 2 and len(s2) == 2 and s1[0] in chrom_sizes_dict.keys() and s2[1] <= chrom_sizes_dict[s1[0]]:
+                    file.write(str(region) + '\n')
+                else:
+                    if s1[0] in chrom_sizes_dict.keys() and s2[1] <= chrom_sizes_dict[s1[0]]:
+                        sys.stderr.write('WARNING: Region {} coordinates not found in the reference sequence. Omitting.\n'.format(region))
+                    else:
+                        sys.stderr.write('WARNING: Region {} is not in the proper format (chrom:start-end). Omitting.\n'.format(region))
 
     faidx(input_fasta_file_path, '-r', regions_file, '-o', output_fasta_file_path)
     
@@ -351,7 +410,7 @@ def create_fasta_file(
 
 
 def true_alignment_match(
-        aligned_segment: pysam.AlignedSegment, needs_liftover: bool = False, contig: str = None, ref_start: int = None,
+        aligned_segment: AlignedSegment, needs_liftover: bool = False, contig: str = None, ref_start: int = None,
         ref_end: int = None, strand: str = None, tag_read: bool = False, only_output_match_label: bool = False, only_output_overlap: bool = False,
         true_reference_sequence_path: str = None, aligned_reference_sequence_path: str = None, liftover_converter = None,
         chain_file_path: str = None, sample: str = None, haplotype: int = None, reference_liftover_converters = None
@@ -390,7 +449,7 @@ def true_alignment_match(
                 liftover_converter = pyliftover.Liftover(chain_file_path)
     
         else:
-            print('could not liftover')
+            sys.stderr.write('Could not liftover\n')
             return
     
         contig = liftover_converter.convert_coordinate(contig, ref_start)[0][0]
@@ -438,9 +497,9 @@ def true_alignment_match(
 
 
 def true_alignment_coordinates(
-        aligned_segment: pysam.AlignedSegment, tag_read: bool = False, contig: str = None, ref_start: int = None,
+        aligned_segment: AlignedSegment, tag_read: bool = False, contig: str = None, ref_start: int = None,
         ref_end: int = None, strand: str = None
-        ) -> object: #Tuple(str,int,int,str):
+        ) -> object:
     """
 
     Args:
@@ -474,7 +533,7 @@ def true_alignment_coordinates(
     return contig, ref_start, ref_end, strand
 
 
-def alignment_type(aligned_segment: pysam.AlignedSegment, tag_read: bool = False) -> str:
+def alignment_type(aligned_segment: AlignedSegment, tag_read: bool = False) -> str:
     """
 
     Args:
